// ============================================================================
// sdl_colorkey_problem.c
// ----------------------------------------------------------------------------
// show problem with color key and SDL_ConvertSurface() in SDL2 2.0.6
// ============================================================================


#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "SDL.h"
#include "SDL_image.h"


#define MIN(a, b)	((a) < (b) ? (a) : (b))
#define MAX(a, b)	((a) > (b) ? (a) : (b))

#define SCREEN_WIDTH	640
#define SCREEN_HEIGHT	480

#define SPRITE_WIDTH	256
#define SPRITE_HEIGHT	64

#define TEST_FILENAME	"test.png"


void PrintColorkey(SDL_Surface *surface, char *text)
{
  Uint32 colorkey;

  if (SDL_GetColorKey(surface, &colorkey) != 0)
  {
    fprintf(stderr, "SDL_GetColorKey() failed: %s\n", SDL_GetError());

    exit(10);
  }

  printf("::: %s: 0x%08lx\n", text, (long unsigned int)colorkey);
}

SDL_Surface *LoadImage(char* filename)
{
  SDL_Surface *surface;

  surface = IMG_Load(filename);
  if (surface == NULL)
  {
    fprintf(stderr, "IMG_Load('%s') failed: %s\n", filename, SDL_GetError());

    exit(10);
  }

  Uint32 colorkey = SDL_MapRGB(surface->format, 0x00, 0x00, 0x00);

  SDL_SetColorKey(surface, SDL_TRUE, colorkey);

  PrintColorkey(surface, "GetColorKey() BEFORE SDL_ConvertSurface()");

  return surface;
}

SDL_Surface *GetNativeSurface(SDL_Surface *surface,
			      SDL_Surface *surface_native)
{
  SDL_PixelFormat format;
  SDL_Surface *new_surface;

  format = *surface_native->format;

  new_surface = SDL_ConvertSurface(surface, &format, 0);

  if (new_surface == NULL)
  {
    fprintf(stderr, "SDL_ConvertSurface() failed: %s\n", SDL_GetError());

    exit(10);
  }

  return new_surface;
}

void DrawImage(SDL_Renderer *renderer, SDL_Texture *texture,
	       SDL_Surface *background, SDL_Surface *surface)
{
  SDL_Rect src_rect, dst_rect;
  int w, h;

  w = MIN(SPRITE_WIDTH, surface->w);
  h = MIN(SPRITE_HEIGHT, surface->h);

  src_rect.x = 0;
  src_rect.y = 0;
  src_rect.w = w;
  src_rect.h = h;

  dst_rect.x = background->w / 2 - w / 2;
  dst_rect.y = background->h / 2 - h / 2;
  dst_rect.w = w;
  dst_rect.h = h;

  SDL_BlitSurface(surface, &src_rect, background, &dst_rect);

  dst_rect.x -= 16;
  dst_rect.y -= 16;

  SDL_BlitSurface(surface, &src_rect, background, &dst_rect);

  SDL_UpdateTexture(texture, NULL, background->pixels, background->pitch);

  SDL_RenderClear(renderer);
  SDL_RenderCopy(renderer, texture, NULL, NULL);
  SDL_RenderPresent(renderer);
}

int main(int argc, char *argv[])
{
  SDL_Window *window;
  SDL_Renderer *renderer;

  if (SDL_CreateWindowAndRenderer(SCREEN_WIDTH, SCREEN_HEIGHT, 0,
				  &window, &renderer) < 0)
  {
    fprintf(stderr, "SDL_CreateWindowAndRenderer() failed: %s\n", SDL_GetError());

    exit(10);
  }

  SDL_Texture *texture = SDL_CreateTexture(renderer,
					   SDL_PIXELFORMAT_ARGB8888,
					   SDL_TEXTUREACCESS_STREAMING,
					   SCREEN_WIDTH, SCREEN_HEIGHT);
  if (texture == NULL)
  {
    fprintf(stderr, "SDL_CreateTexture() failed: %s\n", SDL_GetError());

    exit(10);
  }

  SDL_Surface *background = SDL_CreateRGBSurface(0, SCREEN_WIDTH, SCREEN_HEIGHT,
						 32, 0,0,0, 0);
  if (background == NULL)
  {
    fprintf(stderr, "SDL_CreateRGBSurface() failed: %s\n", SDL_GetError());

    exit(10);
  }

  SDL_Surface *surface = LoadImage(TEST_FILENAME);
  SDL_Surface *new_surface = GetNativeSurface(surface, background);

  PrintColorkey(new_surface, "GetColorKey() AFTER  SDL_ConvertSurface()");

  DrawImage(renderer, texture, background, new_surface);

  Uint8 done = 0;
  SDL_Event event;

  while (!done)
  {
    while (SDL_PollEvent(&event))
    {
      if (event.type == SDL_QUIT ||
	  event.type == SDL_KEYDOWN)
      {
	done = 1;
      }
    }

    SDL_Delay(10);
  }

  exit(0);
}
